/*
#
# Copyright (C) 2011, audin, all right reserved.
# v01: 2011/08 : Added ChaN's xprintf() facility.
# This soft is under LGPL v2.1.  
# http://avr.paslog.jp
#
*/

/*------------------------------------------------------------------------/
/  Universal string handler for user console interface
/-------------------------------------------------------------------------/
/
/  Copyright (C) 2011, ChaN, all right reserved.
/
/ * This software is a free software and there is NO WARRANTY.
/ * No restriction on use. You can use, modify and redistribute it for
/   personal, non-profit or commercial products UNDER YOUR RESPONSIBILITY.
/ * Redistributions of source code must retain the above copyright notice.
/
/-------------------------------------------------------------------------*/

/*
 * Print.h - Base class that provides print() and println()
 * Copyright (c) 2008 David A. Mellis.  All right reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA.
 *
 * Modified 12 April 2011 by Marti Bolivar <mbolivar@leaflabs.com>
 */

#ifndef _PRINT_H_
#define _PRINT_H_

#include "libmaple_types.h"

enum {
    BYTE = 0,
    BIN  = 2,
    OCT  = 8,
    DEC  = 10,
    HEX  = 16
};

#define __XPRINTF__           /* If you need, delete commentout. */
#if defined( __XPRINTF__ )      /*  printf() facility */
#    define _USE_XFUNC_OUT	1	/* 1: Use output functions */
#    define	_CR_CRLF		1	/* 1: Convert \n ==> \r\n in the output char */

#    define _USE_XFUNC_IN	0	/* 1: Use input function */
#    define	_LINE_ECHO		0	/* 1: Echo back input chars in xgets function */

#    ifdef __cplusplus
	     extern "C" {
#    endif /* __cplusplus */
#    include <stdarg.h>
#    ifdef __cplusplus
       }
#    endif /* __cplusplus */

#    define DW_CHAR		sizeof(char)
#    define DW_SHORT	sizeof(short)
#    define DW_LONG		sizeof(long)
#endif

class Print {
  public:
	virtual void write(uint8 ch) = 0;
	virtual void write(const char *str);
	virtual void write(const void *buf, uint32 len);
	void print(char);
	void print(const char[]);
	void print(uint8, int=DEC);
	void print(int, int=DEC);
	void print(unsigned int, int=DEC);
	void print(long, int=DEC);
	void print(unsigned long, int=DEC);
	void print(long long, int=DEC);
	void print(unsigned long long, int=DEC);
	void print(double, int=2);
	void println(void);
	void println(char);
	void println(const char[]);
	void println(uint8, int=DEC);
	void println(int, int=DEC);
	void println(unsigned int, int=DEC);
	void println(long, int=DEC);
	void println(unsigned long, int=DEC);
	void println(long long, int=DEC);
	void println(unsigned long long, int=DEC);
	void println(double, int=2);
  private:
	void printNumber(unsigned long long, uint8);
	void printFloat(double, uint8);

#if defined( __XPRINTF__ ) /*  printf() facility */
  private:
	char *outptr;
	void (*xfunc_out)(unsigned char);	/* Pointer to the output stream */
  public:
	Print(void);
	void xprintf(const char* fmt, ...);
	void printf(const char* fmt, ...);
	void putc(unsigned char c);
	void xputs(const char* str);
	void xputc(char c);
	void xvprintf(
		const char*	fmt,	/* Pointer to the format string */
		va_list arp			/* Pointer to arguments */
		);
	void xsprintf (			/* Put a formatted string to the memory */
							char* buff,			/* Pointer to the output buffer */
							const char*	fmt,	/* Pointer to the format string */
							...					/* Optional arguments */
							);

	void xdprintf (			/* Put a formatted string to the specified device */
							void(*func)(unsigned char),	/* Pointer to the output function */
							const char*	fmt,			/* Pointer to the format string */
							...							/* Optional arguments */
							);
	void put_dump (
		const void* buff,		/* Pointer to the array to be dumped */
		unsigned long addr,		/* Heading address value */
		int len,				/* Number of items to be dumped */
		int width				/* Size of the items (DF_CHAR, DF_SHORT, DF_LONG) */
		);
#endif
	
};

#endif
